/*
 * AbiFestVox - Festival plugin for AbiWord
 * Authors:	Rui Miguel Silva Seabra <rms@1407.org>
 * Inspiration: AikSaurus AbiWord plugin by Jared Davis
 * Copyright  2002 The Free Software Foundation
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "ut_assert.h"
#include "ut_debugmsg.h"
#include "xap_Module.h"
#include "xap_App.h"
#include "xap_Frame.h"
#include "fv_View.h"
#include "ap_Menu_Id.h"
#include "ev_Menu_Actions.h"
#include "ev_Menu.h"
#include "ev_Menu_Layouts.h"
#include "ev_Menu_Labels.h"
#include "ev_EditMethod.h"
#include "xap_Menu_Layouts.h"
#include "fg_Graphic.h"
#include "ie_imp.h"
#include "ie_impGraphic.h"
#include "ie_exp.h"
#include "ie_types.h"

#include "ut_sleep.h"
#include <sys/types.h>  
#include <sys/stat.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/types.h>
#include <signal.h>


using namespace std;

ABI_PLUGIN_DECLARE(AbiFestVox);

bool AbiFestVox_invoke(AV_View* v, EV_EditMethodCallData *d);

static const char* AbiFestVox_MenuLabel = "&Read Aloud Document";
static const char* AbiFestVox_MenuTooltip = "Read aloud the full document";
static const char* AbiFestVox_ContextMenuLabel = "&Read Aloud";
static const char* AbiFestVox_ContextMenuTooltip = "Reads selected text";
static const char* callbackStr = "AbiFestVox_invoke";


//
// AbiFestVox_addToMenus
// -----------------------
//   Adds "Read Aloud" option to AbiWord's Context Menu.
//
static void AbiFestVox_addToMenus() {
	// First we need to get a pointer to the application itself.
	XAP_App *pApp = XAP_App::getApp();


	// Create an EditMethod that will link our method's name with
	// it's callback function.  This is used to link the name to 
	// the callback.
	EV_EditMethod *myEditMethod = new EV_EditMethod(
		callbackStr,            // name of callback function
		AbiFestVox_invoke,   // callback function itself.
		0,                   // no additional data required.
		""         // description -- allegedly never used for anything
	);

	// Now we need to get the EditMethod container for the application.
	// This holds a series of Edit Methods and links names to callbacks.
	EV_EditMethodContainer* pEMC = pApp->getEditMethodContainer();
    
	// We have to add our EditMethod to the application's EditMethodList
	// so that the application will know what callback to call when a call
	// to "AbiFestVox_invoke" is received.
	pEMC->addEditMethod(myEditMethod);
  

	// Now we need to grab an ActionSet.  This is going to be used later
	// on in our for loop.  Take a look near the bottom.
	EV_Menu_ActionSet* pActionSet = pApp->getMenuActionSet();

    
	// We need to go through and add the menu element to each "frame" 
	// of the application.  We can iterate through the frames by doing
	// XAP_App::getFrameCount() to tell us how many frames there are,
	// then calling XAP_App::getFrame(i) to get the i-th frame.

	int frameCount = pApp->getFrameCount();
	XAP_Menu_Factory * pFact = pApp->getMenuFactory();

	//
	// Put it in the context menu.
	//
	XAP_Menu_Id contextID = pFact->addNewMenuAfter(
			"contextText",NULL,"Bullets and &Numbering",
			EV_MLF_Normal
			);
	// Add Label
	pFact->addNewLabel(
			NULL,contextID,
			AbiFestVox_ContextMenuLabel,
			AbiFestVox_ContextMenuTooltip
			);
	// Create the Action that will be called.
	EV_Menu_Action* contextAction = new EV_Menu_Action(
		contextID,              // id that the layout said we could use
		0,                      // no, we don't have a sub menu.
		1,                      // yes, we raise a dialog.
		0,                      // no, we don't have a checkbox.
		callbackStr,            // name of callback function to call.
		NULL,                   // don't know/care what this is for
		NULL                    // don't know/care what this is for
		);

	//
	// Also put it under word Wount in the main menu,
	//
	XAP_Menu_Id mainID = pFact->addNewMenuAfter(
			"Main",NULL,"&Word Count",EV_MLF_Normal
			);
	pFact->addNewLabel(
			NULL,mainID,AbiFestVox_MenuLabel,
			AbiFestVox_MenuTooltip
			);

	// Create the Action that will be called.
	EV_Menu_Action* mainAction = new EV_Menu_Action(
		mainID,                 // id that the layout said we could use
		0,                      // no, we don't have a sub menu.
		1,                      // yes, we raise a dialog.
		0,                      // no, we don't have a checkbox.
		callbackStr,            // name of callback function to call.
		NULL,                   // don't know/care what this is for
		NULL                    // don't know/care what this is for
		);

	// Now what we need to do is add this particular action to the ActionSet
	// of the application.  This forms the link between our new ID that we 
	// got for this particular frame with the EditMethod that knows how to 
	// call our callback function.  

	pActionSet->addAction(contextAction);
	pActionSet->addAction(mainAction);
    
	for(int i = 0;i < frameCount;++i) {
		// Get the current frame that we're iterating through.
		XAP_Frame* pFrame = pApp->getFrame(i);
		pFrame->rebuildMenus();
    }
}

static void AbiFestVox_RemoveFromMenus () {
	// First we need to get a pointer to the application itself.
	XAP_App *pApp = XAP_App::getApp();

	// remove the edit method
	EV_EditMethodContainer* pEMC = pApp->getEditMethodContainer() ;
	EV_EditMethod * pEM = ev_EditMethod_lookup ( callbackStr ) ;
	pEMC->removeEditMethod ( pEM ) ;
	DELETEP( pEM ) ;

	// now remove crap from the menus
	int frameCount = pApp->getFrameCount();
	XAP_Menu_Factory * pFact = pApp->getMenuFactory();

	pFact->removeMenuItem("contextText",NULL,AbiFestVox_MenuLabel);
	pFact->removeMenuItem("contextText",NULL,AbiFestVox_ContextMenuLabel);
	for(int i = 0;i < frameCount;++i) {
		// Get the current frame that we're iterating through.
		XAP_Frame* pFrame = pApp->getFrame(i);
		pFrame->rebuildMenus();
	}
}
    
// -----------------------------------------------------------------------
//
//      Abiword Plugin Interface 
//
// -----------------------------------------------------------------------
    
ABI_FAR_CALL int abi_plugin_register (XAP_ModuleInfo * mi) {
    mi->name = "AbiFestVox";
    mi->desc = "Use this to hear documents or selected portions of text.";
    mi->version = ABI_VERSION_STRING;
    mi->author = "Rui Miguel Silva Seabra <rms@1407.org>";
    mi->usage = "No Usage";
    
    // Add to AbiWord's menus.
    AbiFestVox_addToMenus();
    
    return 1;
}


ABI_FAR_CALL int abi_plugin_unregister (XAP_ModuleInfo * mi) {
    mi->name = 0;
    mi->desc = 0;
    mi->version = 0;
    mi->author = 0;
    mi->usage = 0;

    AbiFestVox_RemoveFromMenus ();

    return 1;
}


ABI_FAR_CALL int abi_plugin_supports_version (
		UT_uint32 major, UT_uint32 minor, UT_uint32 release) {
    return 1; 
}

// -----------------------------------------------------------------------
//
//     AbiFestVox Invocation Code
//
// -----------------------------------------------------------------------


//
// AbiFestVox_invoke
// -------------------
//   This is the function that we actually call to invoke the reader.
//
bool AbiFestVox_invoke(AV_View* v, EV_EditMethodCallData *d) {
	return true;
}
/*
    // Get the current view that the user is in.
    XAP_Frame *pFrame = XAP_App::getApp()->getLastFocussedFrame();
    FV_View* pView = static_cast<FV_View*>(pFrame->getCurrentView());

//
// generate a temp file name...
//
	char szTempFileName[ 2048 ];
	UT_tmpnam(szTempFileName);
	UT_String szTmp = szTempFileName;
	szTmp += ".png";
	
	PT_DocPosition pos = pView->saveSelectedImage((const char *)szTmp.c_str());
//
// Get the initial file status.
//
	if(pos == 0)
	{
		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
		return false;
	}
	struct stat myFileStat;
	int ok = stat(szTmp.c_str(),&myFileStat);
	if(ok < 0)
	{
		UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
		return false;
	}
	time_t mod_time = myFileStat.st_mtime;
//	
// Fire up the gimp...
//
	char * gimpArgs[3];
	gimpArgs[0] = "gimp";
    gimpArgs[1] = const_cast<char *>(szTmp.c_str());
	gimpArgs[2] = NULL;
	UT_sint32 pid;
	printf("!!!!!!!!!!!!!!!!!!!!!SEVIOR: argument 0 = %s \n",gimpArgs[1]);
	if((pid = fork())== 0)
	{
		execvp((const char *)"gimp",gimpArgs);
	}
	int status;
	while (pid != waitpid (pid, &status, WNOHANG)) 
	{
		UT_usleep(10000); // wait 10 milliseconds
		pFrame->nullUpdate();
		ok = stat(szTmp.c_str(),&myFileStat);
		if(ok == 0)
		{ 
			if(myFileStat.st_mtime != mod_time)
			{
				mod_time = myFileStat.st_mtime;
				UT_usleep(100000); // wait 100 milliseconds
				ok = stat(szTmp.c_str(),&myFileStat);
				while(mod_time != myFileStat.st_mtime)
				{
					mod_time = myFileStat.st_mtime;
					ok = stat(szTmp.c_str(),&myFileStat);
					UT_usleep(100000); // wait 100 milliseconds
				}
//
// OK replace the current image with this.
//
				IEGraphicFileType iegft = IEGFT_Unknown;
				IE_ImpGraphic *pIEG;
				FG_Graphic* pFG;
		
				UT_Error errorCode;
		
				errorCode = IE_ImpGraphic::constructImporter(szTmp.c_str(),iegft, &pIEG);
				if(errorCode)
				{
					pFrame->showMessageBox("Error constructing importer. Could not put image back into Abiword", XAP_Dialog_MessageBox::b_O,XAP_Dialog_MessageBox::a_OK);
					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
					goto Cleanup;
				}
				errorCode = pIEG->importGraphic(szTmp.c_str(), &pFG);
				if(errorCode)
				{
					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
					pFrame->showMessageBox("Error making pFG. Could not put image back into Abiword", XAP_Dialog_MessageBox::b_O,XAP_Dialog_MessageBox::a_OK);
					DELETEP(pIEG);
					goto Cleanup;
				}
				DELETEP(pIEG);
				pView->cmdUnselectSelection();
				pView->setPoint(pos);
				pView->extSelHorizontal(true, 1); // move point forward one
				errorCode = pView->cmdInsertGraphic(pFG,szTmp.c_str());
				if (errorCode)
				{
					pFrame->showMessageBox("Could not put image back into Abiword", XAP_Dialog_MessageBox::b_O,XAP_Dialog_MessageBox::a_OK);
					UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
					DELETEP(pFG);
					goto Cleanup;
				}
				DELETEP(pFG);
//
// Reselect the image
//
				pView->setPoint(pos);
				pView->extSelHorizontal(true, 1); // move point forward one
			}
		}
	}
//
// Now delete the tempfile
//
	ok = unlink(szTempFileName);
	ok = unlink(szTmp.c_str());
//
// Done!
//    
    return true;
//
// Something went wrong.
//
 Cleanup: 
	ok = unlink(szTempFileName);
	ok = unlink(szTmp.c_str());
//
// Kill the gimp.
//
	kill(pid,9);
	return false;
}

*/

